unit MyLibraryInterface;

{$mode objfpc}{$H+}

{*******************************************************************************
*          Libraray Interface for accessing Lazarus LCL Objects in a DLL       *
*                                                                              *
*          Latest changes:  2012-01-04   by Breakoutbox                        *
********************************************************************************
* Hints:                                                                       *
*   -  Take a look at the  INITIALIZATION  and  FINALIZATION  sections !       *
*      There we init VARs and also the DLL is loaded and unloaded there !      *
*******************************************************************************}

interface

uses
  Classes, SysUtils, StdCtrls, DynLibs, Dialogs, rwincontrol, rwincontrolserver;


const
  LibName = 'TestLibrary.dll';    // Windows only ..


const
  //-----------------------------------------------------
  // This section contains all the names of the functions
  // exported by the DLL (as const strings) :
  sCreateBtn = 'CreateBtn';
  sCreateLbl = 'CreateLbl';


type
  //-----------------------------------------------------
  // define Prototype functions for Your EXE project
  // according to the functions You have realized in the DLL :
  TDll_create_TButton = function( intr:TRWinControlInterface; Parent:TRHandle):TButton;
  TDll_create_TLabel = function( intr:TRWinControlInterface; Parent:TRHandle):TLabel;


var
  //-----------------------------------------------------
  // In this example we load the library here (in this unit)
  // and also we place the following VARs here
  // to hide the DYNLIB.PAS functions from the MainForm.pas :
  lib : TLibHandle;
  FNcreateButton : TDll_create_TButton;
  FNcreateLabel : TDll_create_TLabel;



  //-------------------------------------------------------------------
  // Now this are the functions that create the Objects from / by DLL :
function DLLcreateTButton( aParent : TRHandle):TButton;
function DLLcreateTLabel( aParent : TRHandle):TLabel;


implementation

{ create a Button from our DLL : }
function DLLcreateTButton( aParent : TRHandle):TButton;
begin
  if lib = 0  // if the DLL is not initialzed  we cannot create the Object !
    then result:= NIL
    else
      begin
        RegisterClass( TButton);
        FNcreateButton:= TDll_create_TButton( GetProcAddress( lib, sCreateBtn));
        result:= FNcreateButton( FillRWinControlInterface, aParent);
      end;
end;

{ create a Label from our DLL : }
function DLLcreateTLabel( aParent : TRHandle):TLabel;
begin
  if lib = 0
    then result:= NIL
    else
      begin
        RegisterClass( TLabel);
        FNcreateLabel:= TDll_create_TLabel( GetProcAddress( lib, sCreateLbl));
        result:= FNcreateLabel( FillRWinControlInterface, aParent);
      end;
end;



initialization
  { init our Function Pointers to prevent errors : }
  FNcreateButton:= NIL;
  FNcreateLabel:= NIL;
  { load the library here - for the whole runtime of the program : }
  lib:= LoadLibrary( LibName);


finalization
  { unload the library on closing the MainForm : }
  FreeLibrary( lib);


end.

